//
// PIRProcessor.h
//
// This file contains declarations for the PIRProcessor class.
//
// These classes are part of the Bioinformatics Template Library (BTL).
//
// Copyright (C) 1997,1998 Birkbeck College, Malet Street, London WC1E 7HX, U.K.
// (classlib@mail.cryst.bbk.ac.uk)
// 
// This library is free software; you can
// redistribute it and/or modify it under the terms of
// the GNU Library General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.  This library is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
///////////////////////////////////////////////////////////////////////////

#if !defined(BTL_PIRPROCESSOR_H)
#define BTL_PIRPROCESSOR_H

#if !defined(SGI_CC)
#include <string>
#include <vector>
#else
#include <mstring.h>
#include <vector.h>
#endif

#include <fstream.h>

#if !defined(SGI_CC) && !defined(GCC_272)
using namespace std;
#endif

/**#: [Description ="This class is a simple PIR sequence file processor. It is
    not complete and is only included for use by the demo programs that come
    with this library. It can parse a PIR format containing any number
    of sequences and provide the data as a vector<string> or as individual
    strings."]
    [Summary = "simple class for parsing PIR format sequence files"] 
    [Authors = "W.R.Pitt"]
    [Files = "<A HREF=./btl/PIRProcessor.h>PIRProcessor.h</A>,
              <A HREF=./btl/PIRProcessor.cpp>PIRProcessor.cpp</A>"]
    [Dependencies="none"]

*/

class BTL_PIRProcessor
{
private:

    vector<string> seqs;
    
	    	/**#: [Hidden] */
    void 
    OpenFile(const char* fileName, ifstream& fileStream);
    
	    	/**#: [Hidden] */
    int 
    HowManySeqs(const char* fileName, ifstream& fileStream);

public:

	    	/**#: [Description="Default constructor (does nothing)"] */
    BTL_PIRProcessor() {}
    
	    	/**#: [Description="Return true if no sequences have been
	    	       read in, false otherwise."] */
    bool
    empty() const { return seqs.empty(); } 
    
	    	/**#: [Description="Read a file with a given name. If this 
	    	       function is called more than once, the new sequences
	    	       are added to those already there."] */
    void
    ReadFile(const char *fileName);
    
	    	/**#: [Description="Return any sequences that been read in."] */
    vector<string>&
    Seqs() { return seqs; }
    
	    	/**#: [Description="Return a single sequence, given a valid
	    	       index number"] */
    string&
    Seq(const unsigned int iseq) 
    {   
    	if (iseq>=seqs.size()) 
    	{
    	    cerr << "!!Invalid sequence no." << endl;
    	    exit(1);
    	}
    	return seqs[iseq];
    }
};

#endif
