//
// PPMProcessor.h
//
// This file contains the BTL_PPMProcessor class.
//
// These classes are part of the Bioinformatics Template Library (BTL).
//
// Copyright (C) 1997,1998 Birkbeck College, Malet Street, London WC1E 7HX, U.K.
// (classlib@mail.cryst.bbk.ac.uk)
// 
// This library is free software; you can
// redistribute it and/or modify it under the terms of
// the GNU Library General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.  This library is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
///////////////////////////////////////////////////////////////////////////

#if !defined (BTL_PPMProcessor_H)
#define BTL_PPMProcessor_H

#include <fstream.h>
#include <iostream.h>

#if !defined(SGI_CC)
#include <vector>
#else
#include <vector.h>
#endif

#if !defined(SGI_CC) && !defined(GCC_272)
using namespace std;
#endif

/**#: [Description ="This class is a simple ascii PPM image file processor. It 
    is not complete and is only included for use by the demo programs that come
    with this library."] 
    [Summary = "simple class for parsing ascii PPM format image files"] 
    [Authors = "W.R.Pitt"]
    [Friends = "ostream operator"]
    [Files = "<A HREF=./btl/PPMProcessor.h>PPMProcessor.h</A>,
              <A HREF=./btl/PPMProcessor.cpp>PPMProcessor.cpp</A>"]
    [Dependencies="none"]

*/


class BTL_PPMProcessor
{
private:

    unsigned int width, height;
    vector<double> imageData;
    
	    	/**#: [Hidden] */
    void 
    OpenFile(const char* fileName, ifstream& fileStream);

	    	/**#: [Hidden] */
    void 
    GetDimensions(ifstream& fileStream);

	    	/**#: [Hidden] */
    void 
    GetImageData(unsigned int size, ifstream& fileStream);
    
public:
    
	    	/**#: [Description="Read in a file with a given name."] */
    void
    ReadFile(const char *fileName);
    
	    	/**#: [Description="Get image data."] */
    vector<double>& 
    ReadImageData() { return imageData; }

	    	/**#: [Description="Write image data."] */
    void
    WriteImageData(const vector<double>& idata, 
                   const unsigned int iwidth,
                   const unsigned int iheight); 

	    	/**#: [Description="Read width of image."] */
    unsigned int
    ReadWidth() const { return width; }
     
	    	/**#: [Description="Read image height."] */
    unsigned int
    ReadHeight() const { return height; }
     
    friend ostream&
    operator<<(ostream &os, const BTL_PPMProcessor &v);
    
};

#endif
