//
// XYZProcessor.cpp
//
// This file contains definitions for the XYZProcessor class.
//
// These classes are part of the Bioinformatics Template Library (BTL).
//
// Copyright (C) 1997,1998 Birkbeck College, Malet Street, London WC1E 7HX, U.K.
// (classlib@mail.cryst.bbk.ac.uk)
// 
// This library is free software; you can
// redistribute it and/or modify it under the terms of
// the GNU Library General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.  This library is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
///////////////////////////////////////////////////////////////////////////

#include "btl/XYZProcessor.h"

ostream& operator<<(ostream& os, const BTL_XYZProcessor& x)
{
    os << x.title << "\n\n";

    unsigned int nAtoms = x.coords.size()/3;
    for (unsigned int i=0, j=0, k=0; i<nAtoms; i++)
    {
    	os << setw(3) << i << " : " 
    	   << x.elements[j++] 
    	   << x.elements[j++]    	   
    	   << " " << setw(10) << x.coords[k++] 
    	   << " " << setw(10) << x.coords[k++]
    	   << " " << setw(10) << x.coords[k++]
    	   << '\n';
    }
    os << '\n';
    return os;
}

void BTL_XYZProcessor::OpenFile(const char* fileName, ifstream& fileStream)
{
    fileStream.open(fileName);

    // check that file can be opened
    if (!fileStream.good())
    {
        cerr << "\n\nFile " << fileName << " cannot be opened" << endl;
        exit(1);
    }
    
}
void BTL_XYZProcessor::ReadFile(const char *fileName)
{ 
    ifstream fin;
    OpenFile(fileName,fin);

    const short MAXLINE = 512;
    char line[MAXLINE];

    // Number of coords
    //
    if (!fin.getline(line,MAXLINE)) return;
    unsigned int nAtoms = atoi(line);
    
    elements.erase(elements.begin(),elements.end());
    elements.reserve(nAtoms*2);
    coords.erase(coords.begin(),coords.end());
    coords.reserve(nAtoms*3);
    
    // Title
    //
    if (!fin.getline(line,MAXLINE)) return;
    title = string(line,MAXLINE);
    
    // Read coords
    //
    char word[50];
    for (unsigned int i=0; i<nAtoms; i++)
    {
    	if (!fin.getline(line,MAXLINE)) break;
    	istrstream instr(line,MAXLINE);
    	word[0] = 0;
    	instr >> word;
    	elements.push_back(line[0]);
    	elements.push_back(line[1]);
    	for (int j=0; j<3; j++)
    	{
    	    word[0] = 0;
    	    instr >> word;
    	    coords.push_back(atof(word));
    	}    
    }
} 

