   
#include "btl/XYZProcessor.h"
#include "btl/GraphWithEdges.h"
//#include "btl/DFSIterator.h"
//#include <vector>
#include <math.h>

class Atom
{
private:

    int     id;
    char    element[2];
    float   x,y,z;
    
public:

    friend class Molecule;

    Atom() {}

    friend ostream& 
    operator<<(ostream& os, const Atom& a);
};

class Molecule
{
public:

    typedef BTL_Graph6< vector<Atom>, vector<float> > AtomsAndBonds;

private:

    AtomsAndBonds atomsAndBonds;
    
public:
  
    Molecule(BTL_XYZProcessor& p);
       
    void 
    CreateBonds();
    
    AtomsAndBonds&
    Graph() { return atomsAndBonds; }

    friend ostream& 
    operator<<(ostream& os, const Molecule& m);
};    

int main(int argc, char* argv[])
{
    if (argc != 2)
    {
        cerr << "Usage: MolGraph inputXYZFile" << endl;
        exit(1);
    }

    BTL_XYZProcessor p;
    p.ReadFile(argv[1]);
    if (p.empty()) exit(1);
    
    Molecule molecule(p);
    
    molecule.CreateBonds();

    cout << molecule;
    
    return 0;
}

ostream& operator<<(ostream& os, const Atom& a)
{
    os << a.id << " " 
       << a.element[0] << a.element[1] << " "
       << a.x << " "
       << a.y << " "
       << a.z << '\n';
    return os;
}   

Molecule::Molecule(BTL_XYZProcessor& p)
{
    BTL_XYZProcessor::CoordStore &coords = p.Coords();
    BTL_XYZProcessor::ElementStore &elements = p.Elements();
    
    Atom atom;
    int nAtoms = coords.size()/3;
    for (int i=0, j=0, k=0; i<nAtoms; i++)
    {
    	atom.id = i+1;
    	atom.element[0] = elements[j++];
    	atom.element[1] = elements[j++];
    	atom.x = coords[k++];
    	atom.y = coords[k++];
    	atom.z = coords[k++];
    	atomsAndBonds.AddVertex(atom);
    }
}
    
void Molecule::CreateBonds()
{
    if (atomsAndBonds.size() < 2) return;

    const float distCutoff = 1.7, cutSqr = distCutoff*distCutoff;
    float dx, dy, dz, distSqr;
    
    typedef AtomsAndBonds::Vertex Vertex;
    AtomsAndBonds::VertexIterator i,j,lastButOne;
    lastButOne=atomsAndBonds.end_vertex();
    lastButOne--;
    for (i=atomsAndBonds.begin_vertex(); i!=lastButOne; i++)    
        for (j=i, j++; j!=atomsAndBonds.end_vertex(); j++)
        {
	    const Vertex& v1 = *i;
	    const Vertex& v2 = *j;
            const Atom& atom1 = *(v1.GetIterator());
            const Atom& atom2 = *(v2.GetIterator());
            
            dx = atom1.x - atom2.x;
            dy = atom1.y - atom2.y;
            dz = atom1.z - atom2.z;
          
            distSqr = dx*dx + dy*dy + dz*dz;
            
            if (distSqr < cutSqr)
                atomsAndBonds.AddEdge(i,j,fsqrt(distSqr));
        }
}

ostream& operator<<(ostream& os, const Molecule& m)
{
    os << m.atomsAndBonds;
    return os;
}   
