#if !defined(SGI_CC)
#include <string>
#include <vector>
#include <list>
#include <set>
#include <map>
#else
#include <mstring.h>
#include <vector.h>
#include <list.h>
#include <set.h>
#include <map.h>
#endif

#include <iostream.h>
#include "btl/Graph.h"
#include "btl/DFSIterator.h"

#if !defined(SGI_CC) && !defined(GCC_272)
using namespace std;
#endif
  
template <class T1, class T2>
ostream& operator<<(ostream& os, const pair<T1,T2>& x)
{ os << " (" << x.first << "," << x.second << ") "; return os; }

template <class T1, class T2, class T3>
ostream& operator<<(ostream& os, const map<T1,T2,T3>& x)
{ 
    typedef BTL_TYPENAME map<T1,T2,T3>::const_iterator iterator;
    for (iterator i=x.begin(); i!=x.end(); i++)
    	os << *i;
    os << '\n'; 
    return os; 
}

typedef map< int,int,less<int> >    LinkData;
typedef vector<int> 	    	    VertexIndxs;

template<class Graph, class VertexData, class ChangeData>
void Test1(Graph& g, VertexData& vertexData, LinkData& links, 
           VertexIndxs& vertexRemoves, LinkData& linkRemoves,
           ChangeData& change)
{

    // Add some vertices
    //
    typedef BTL_TYPENAME Graph::VertexIterator VertexIterator;
    vector<VertexIterator> vertexIters;
    typedef BTL_TYPENAME VertexData::iterator VertexDataIterator;
    for (VertexDataIterator i=vertexData.begin(); i!=vertexData.end(); i++)
    	vertexIters.push_back(g.AddVertex(*i));
    
    // Add some links between these vertices
    //
    typedef BTL_TYPENAME LinkData::iterator LinkIterator;	
    for (LinkIterator i=links.begin(); i!=links.end(); i++)
    	g.AddLink( vertexIters[(*i).first], vertexIters[(*i).second] );

    // Remove some vertices
    //
    typedef BTL_TYPENAME VertexIndxs::iterator RemoveIterator;
    for (RemoveIterator i=vertexRemoves.begin(); i!=vertexRemoves.end(); i++)
    	g.RemoveVertex(vertexIters[*i]);

    // Remove some links
    //
    for (LinkIterator i=linkRemoves.begin(); i!=linkRemoves.end(); i++)
    	g.RemoveLink( vertexIters[(*i).first], vertexIters[(*i).second] );
    	
    // Add all vertices back in (with maps and sets this can result in
    // multiple vertices referencing the same data)
    //	
    vertexIters.erase(vertexIters.begin(),vertexIters.end());
    
    for (VertexDataIterator i=vertexData.begin(); i!=vertexData.end(); i++)
    	vertexIters.push_back(g.AddVertex(*i));
    
    // Add links again
    //	       	
    for (LinkIterator i=links.begin(); i!=links.end(); i++)
    	g.AddLink( vertexIters[(*i).first], vertexIters[(*i).second] );
    	
    	
    // Make a change to the data in one vertex
    //
    BTL_TYPENAME Graph::Vertex& vert = *(vertexIters[change.first]);
    *(vert.GetIterator()) = change.second;
    
    cout << "\n+++++++++++++ DATA ++++++++++++\n";

    for(BTL_TYPENAME Graph::iterator i=g.begin(); i!=g.end(); i++)
    	cout << *i << " ";
	
    cout << "\n%%%%%%%%%%%% GRAPH %%%%%%%%%%%%\n";
    
    cout << g;
}

template<class Graph, class VertexData>
void Test2(Graph& g, VertexData& vertexData, LinkData& links, 
           VertexIndxs& vertexRemoves, LinkData& linkRemoves)
{

    // Add some vertices
    //
    typedef BTL_TYPENAME Graph::VertexIterator VertexIterator;
    vector<VertexIterator> vertexIters;
    typedef BTL_TYPENAME VertexData::iterator VertexDataIterator;
    for (VertexDataIterator i=vertexData.begin(); i!=vertexData.end(); i++)
    	vertexIters.push_back(g.AddVertex(*i));
    
    // Add some links between these vertices
    //	
    typedef BTL_TYPENAME LinkData::iterator LinkIterator;	
    for (LinkIterator i=links.begin(); i!=links.end(); i++)
    	g.AddLink( vertexIters[(*i).first], vertexIters[(*i).second] );

    // Remove some vertices
    //
    typedef BTL_TYPENAME VertexIndxs::iterator RemoveIterator;
    for (RemoveIterator i=vertexRemoves.begin(); i!=vertexRemoves.end(); i++)
    	g.RemoveVertex(vertexIters[*i]);

    // Remove some links
    //
    for (LinkIterator i=linkRemoves.begin(); i!=linkRemoves.end(); i++)
    	g.RemoveLink( vertexIters[(*i).first], vertexIters[(*i).second] );
    	
    // Add all vertices back in (with maps and sets this can result in
    // multiple vertices referencing the same data)
    //	
    vertexIters.erase(vertexIters.begin(),vertexIters.end());
    
    for (VertexDataIterator i=vertexData.begin(); i!=vertexData.end(); i++)
    	vertexIters.push_back(g.AddVertex(*i));
    
    // Add links again
    //	       	
    for (LinkIterator i=links.begin(); i!=links.end(); i++)
    	g.AddLink( vertexIters[(*i).first], vertexIters[(*i).second] );
    	    	
    
    cout << "\n+++++++++++++ DATA ++++++++++++\n";

    for(BTL_TYPENAME Graph::iterator i=g.begin(); i!=g.end(); i++)
    	cout << *i << " ";
	
    cout << "\n%%%%%%%%%%%% GRAPH %%%%%%%%%%%%\n";
    
    cout << g;
}

template<class Graph, class VertexData>
void Test3(Graph& g, VertexData& vertexData, LinkData& links, 
    	   VertexIndxs& starts)
{
    // Add some vertices
    //
    typedef BTL_TYPENAME Graph::VertexIterator VertexIterator;
    vector<VertexIterator> vertexIters;
    typedef BTL_TYPENAME VertexData::iterator VertexDataIterator;
    for (VertexDataIterator i=vertexData.begin(); i!=vertexData.end(); i++)
    	vertexIters.push_back(g.AddVertex(*i));
    
    // Add some links between these vertices
    //	
    typedef BTL_TYPENAME LinkData::iterator LinkIterator;	
    for (LinkIterator i=links.begin(); i!=links.end(); i++)
    	g.AddLink( vertexIters[(*i).first], vertexIters[(*i).second] );

    // Print the connected sub-graphs starting from different vertices.
    //
    typedef BTL_TYPENAME VertexIndxs::iterator IndxIterator;
    for (IndxIterator i=starts.begin(); i!=starts.end(); i++)
    {
    	BTL_ConstDFSIterator<Graph> di(vertexIters[*i]);
	cout << "Depth first search:\n";
    	while (!di)
    	{
    	    cout << *di++ << " ";
    	}
	cout << "\n\n";
    }
}    


int main()
{    
    
    typedef BTL_TYPENAME LinkData::value_type Link;
    LinkData linkData;
    linkData.insert(Link(0,1));
    linkData.insert(Link(1,2));
    linkData.insert(Link(2,3));
    linkData.insert(Link(3,4));
    linkData.insert(Link(4,1));
    
    VertexIndxs removeVertexData;
    removeVertexData.push_back(2);
    removeVertexData.push_back(3);
    
    VertexIndxs searchStarts;
    searchStarts.push_back(0);
    searchStarts.push_back(3);

    LinkData removeLinksData;
    removeLinksData.insert(Link(4,1));

    typedef set<int, less<int> > Container1;
    typedef BTL_Graph1<Container1> Graph1;
    
    Container1 intVertexData;
    intVertexData.insert(10);	
    intVertexData.insert(9);	
    intVertexData.insert(8);
    intVertexData.insert(7);
    intVertexData.insert(6);

    Graph1 g1;

    Test2(g1,intVertexData,linkData,removeVertexData,removeLinksData);

    typedef vector<float> Container2;
    typedef BTL_Graph2<Container2> Graph2;

    Container2 floatVertexData;
    floatVertexData.push_back(1.1);
    floatVertexData.push_back(2.1);
    floatVertexData.push_back(3.1);
    floatVertexData.push_back(4.1);
    floatVertexData.push_back(5.1);

    pair<int,float> floatContentChange(0,99.9);

    Graph2 g2;

    Test1(g2,floatVertexData,linkData,removeVertexData,removeLinksData,
          floatContentChange);

    typedef map<char,float,less<char> > Container3;
    typedef BTL_TYPENAME Container3::value_type value_type;
    typedef BTL_Graph1<Container3> Graph3;

    Container3 mapVertexData;
    value_type item1('e',1.1);
    value_type item2('d',2.1);
    value_type item3('c',3.1);
    value_type item4('b',4.1);
    value_type item5('a',5.1);
    mapVertexData.insert(item1);
    mapVertexData.insert(item2);
    mapVertexData.insert(item3);
    mapVertexData.insert(item4);
    mapVertexData.insert(item5);

    Graph3 g3;

    Test2(g3,mapVertexData,linkData,removeVertexData,removeLinksData);

    typedef list<char> Container4;
    typedef BTL_Graph1<Container4> Graph4;

    Container4 charVertexData;
    charVertexData.push_back('e');
    charVertexData.push_back('d');
    charVertexData.push_back('c');
    charVertexData.push_back('b');
    charVertexData.push_back('a');

    pair<int,char> charContentChange(0,'z');

    Graph4 g4;

    Test1(g4,charVertexData,linkData,removeVertexData,removeLinksData,
    	  charContentChange);

    typedef list<double> Container5;
    typedef BTL_Graph1<Container5> Graph5;

    Container5 dbleVertexData;
    dbleVertexData.push_back(1.1111111111111);
    dbleVertexData.push_back(2.2222222222222);
    dbleVertexData.push_back(3.3333333333333);
    dbleVertexData.push_back(4.4444444444444);
    dbleVertexData.push_back(5.5555555555555);

    pair<int,double> dbleContentChange(0,9.9999999999999);

    Graph5 g5;

    Test1(g5,dbleVertexData,linkData,removeVertexData,removeLinksData,
    	  dbleContentChange);

    Test3(g1,intVertexData,linkData,searchStarts);

    return 0;
}
