#if !defined(SGI_CC)
#include <string>
#include <vector>
#include <list>
#include <map>
#else
#include <mstring.h>
#include <vector.h>
#include <list.h>
#include <map.h>
#endif

#include <iostream.h>
#include "btl/GraphWithEdges.h"

#if !defined(SGI_CC) && !defined(GCC_272)
using namespace std;
#endif

template <class T1, class T2>
ostream& operator<<(ostream& os, const pair<T1,T2>& x)
{ os << " (" << x.first << "," << x.second << ") "; return os; }

template <class T1, class T2, class T3>
ostream& operator<<(ostream& os, const map<T1,T2,T3>& x)
{
    for (map<T1,T2,T3>::const_iterator i=x.begin(); i!=x.end(); i++)
    	os << *i;
    os << '\n';
    return os;
}

typedef map< int,int,less<int> >    Links;
typedef vector<int> 	    	    Removes;

template<class Graph, class VertexData, class EdgeData>
void Test(Graph& g, VertexData& vertexData, EdgeData& edgeData,
    	   Links& links, Removes& vertexRemoves, Removes& edgeRemoves)
{

    // Add some vertices
    //
    typedef BTL_TYPENAME Graph::VertexIterator VertexIterator;
    vector<VertexIterator> vertexIters;
    typedef BTL_TYPENAME VertexData::iterator VertexDataIterator;
    for (VertexDataIterator i=vertexData.begin(); i!=vertexData.end(); i++)
    	vertexIters.push_back(g.AddVertex(*i));

    // Add some links between these vertices
    //
    typedef BTL_TYPENAME Graph::EdgeIterator EdgeIterator;
    typedef BTL_TYPENAME Links::iterator LinksIterator;
    vector<EdgeIterator> edgeIters;
    typedef BTL_TYPENAME EdgeData::iterator EdgeDataIterator;
    EdgeDataIterator edge = edgeData.begin();
    for (LinksIterator i=links.begin(); i!=links.end(); i++, edge++)
    	edgeIters.push_back(g.AddEdge( vertexIters[(*i).first],
	                               vertexIters[(*i).second],
				       *edge ));

    // Remove some vertices
    //
    typedef BTL_TYPENAME Removes::iterator RemovesIterator;
    for (RemovesIterator i=vertexRemoves.begin(); i!=vertexRemoves.end(); i++)
    	g.RemoveVertex(vertexIters[*i]);

    // Remove some edges
    //
    for (RemovesIterator i=edgeRemoves.begin(); i!=edgeRemoves.end(); i++)
    	if (!g.RemoveEdge(edgeIters[*i])) cout << "\n!An edge not removed.\n";


    // Add all the original vertices and edges back again
    //
    vertexIters.erase(vertexIters.begin(),vertexIters.end());
    for (VertexDataIterator i=vertexData.begin(); i!=vertexData.end(); i++)
    	vertexIters.push_back(g.AddVertex(*i));
	
    edgeIters.erase(edgeIters.begin(),edgeIters.end());
    edge = edgeData.begin();
    for (LinksIterator i=links.begin(); i!=links.end(); i++, edge++)
    	edgeIters.push_back(g.AddEdge( vertexIters[(*i).first],
	                               vertexIters[(*i).second],
				       *edge ));
    
    cout << "\n+++++++++++++ VERTEX DATA ++++++++++++\n";

    for(BTL_TYPENAME Graph::iterator i=g.begin(); i!=g.end(); i++)
    	cout << *i << " ";
	
    cout << "\n-------------- EDGE DATA -------------\n";

    for(BTL_TYPENAME Graph::EdgeDataIterator i=g.BeginEdgeData(); i!=g.EndEdgeData(); i++)
    	cout << *i << " ";

    cout << "\n%%%%%%%%%%%% GRAPH %%%%%%%%%%%%\n";
    
    cout << g;
}

int main()
{    
    typedef set<int, less<int> > SetOfInt;
    typedef vector<char> VectorOfChar;
    typedef vector<float> VectorOfFloat;
    typedef list<float> ListOfFloat;
    typedef Links::value_type Link;

    SetOfInt setOfInt1;
    setOfInt1.insert(10);
    setOfInt1.insert(9);
    setOfInt1.insert(8);
    setOfInt1.insert(7);
    setOfInt1.insert(6);

    SetOfInt setOfInt2;
    setOfInt2.insert(100);
    setOfInt2.insert(90);
    setOfInt2.insert(80);
    setOfInt2.insert(70);
    setOfInt2.insert(60);

    VectorOfChar vertexOfChar1;
    vertexOfChar1.push_back('z');
    vertexOfChar1.push_back('y');
    vertexOfChar1.push_back('x');
    vertexOfChar1.push_back('w');
    vertexOfChar1.push_back('v');

    VectorOfChar vertexOfChar2;
    vertexOfChar2.push_back('a');
    vertexOfChar2.push_back('b');
    vertexOfChar2.push_back('c');
    vertexOfChar2.push_back('d');
    vertexOfChar2.push_back('e');

    ListOfFloat listOfFloat;
    listOfFloat.push_back(5.5);
    listOfFloat.push_back(4.4);
    listOfFloat.push_back(3.3);
    listOfFloat.push_back(2.2);
    listOfFloat.push_back(1.1);

    VectorOfFloat vectorOfFloat;
    vectorOfFloat.push_back(5.5);
    vectorOfFloat.push_back(4.4);
    vectorOfFloat.push_back(3.3);
    vectorOfFloat.push_back(2.2);
    vectorOfFloat.push_back(1.1);

    string string1, string2;
    string1 = "vertices";
    string2 = "edges";

    Links links;
    links.insert(Link(0,1));
    links.insert(Link(1,2));
    links.insert(Link(2,3));
    links.insert(Link(3,4));
    links.insert(Link(4,1));

    Removes vertexRemoves;
    vertexRemoves.push_back(2);
    vertexRemoves.push_back(3);
    
    Removes edgeRemoves;
    edgeRemoves.push_back(4);

    // A graph where vertices and edges values are ints and stored in a set
    //
    BTL_Graph3<SetOfInt> g1;
    Test(g1, setOfInt1, setOfInt2, links, vertexRemoves,edgeRemoves);

    // A graph where vertices and edges values are chars and stored in a vector
    //
    BTL_Graph4<VectorOfChar> g2;    
    Test(g2, vertexOfChar1, vertexOfChar2, links, vertexRemoves, edgeRemoves);

    
    // A graph where vertices values are ints stored in a set and edges values
    // are floats and stored in a list
    //
    BTL_Graph5<SetOfInt, ListOfFloat> g3;
    Test(g3, setOfInt1, listOfFloat, links, vertexRemoves, edgeRemoves);
	
    // Old version of string class doesn't have the erase function
    //
#if !defined(GCC_272)

    // A graph where vertices and edges values are chars and stored in a string
    //
    BTL_Graph4<string> g4;
    Test(g4, string1, string2, links, vertexRemoves, edgeRemoves);

    // A graph where vertices values are chars stored in a string and edges
    // values are floats and stored in a vector
    //
    BTL_Graph6<string,VectorOfChar> g5;
    Test(g5, string1, vertexOfChar1, links, vertexRemoves, edgeRemoves);

#endif

    return 0;
}
